if __name__ == "__main__":
    import sys
    sys.path.append("/home/erwan/eve/test/asm/src")
import asmat.generation as generation
import asmat.dependencies as files
import asmat.const as const
import os
import asmat.validation as validation
from asmat.analysis.analyze import analyze

"""
        Assembly Analysis and Testing

This is the main file of the ASMAT.
ASMAT is originally made for validating EVE assembly code.
The scope of this module has increase to be used on other c++ libraries.

The 3 main usefulness of this module are:
    - generation
    - validation
    - analysis
"""



def flags_to_command_line(flags:list):
    fl = []
    for i in flags:
        fl.append('-' + i)
    return fl



def options_to_dict(options:list):
    """Get dictionary from args list

    Args:
        options (list): List of options given by os.argv. The first element of the list should be the name of the file.

    Raises:
        Exception: If option `-input` is not followed by files names
        Exception: If the option doesn't exist

    Returns:
       dict : dictionary with each option value
    """
    opt = const.OPTIONS.copy()

    i = 1
    while i < len(options):
        if options[i] == '-m':
            lst = []
            while i+1 < len(options) and options[i+1][0] != '-':
                lst.append(options[i+1])
                i+=1
            if len(lst) == 0:
                raise Exception("Parameter missing for option -m")
            for j in lst:
                if j not in const.ARCH:
                    raise Exception(f"Invalid architecture name '{j}'")
            opt['arch'] = lst
        elif options[i] == '-l':
            opt['log'] = True
        elif options[i] == "-d":
            opt['deep'] = True
        elif options[i] == "-a" or options[i] == "--analysis":
            opt['analyze'] = True
            opt['validate'] = False
            opt['generate'] = False
        elif options[i] == '-t':
            opt['keep_tmp'] = True
        elif options[i] == '-g':
            opt['generate'] = True
            opt['validate'] = False
            opt['analyze'] = False
        elif options[i] == '-i':
            opt["instruction_comparison"] = True
        elif options[i] == '-v':
            opt['verbose'] = True
        elif options[i] == "--version":
            print(const.VERSION)
            opt['validate'] = False
            opt['generate'] = False
            opt['analyze'] = False
        elif options[i] == '--fatal':
            opt['exception'] = True
        elif options[i] == '--functionsperfile' or options[i] == '-S':
            opt['limit_per_file'] = int(options[i+1])
            i+=1
        elif options[i] == '--ref':
            opt['ref_path'] = options[i+1]
            i+=1
        elif options[i] == '--reset':
            opt['validate'] = False
            opt['generate'] = False
            opt['analyze'] = False
            e = input("All references files will be deleted, do you confirm ? (Y/n)")
            if e == 'Y':
                files.reset()
            break
        elif options[i] == '--build':
            opt['validate'] = False
            opt['generate'] = False
            opt['analyze'] = False
            files.build_dependencies()
            break
        elif options[i] == "--disassembler" or options[i] == "-D":
            if options[i+1] == 'objdump' or options[i+1] == 'standard':
                opt['disassembler'] = options[i+1]
                i+=1
            else:
                raise Exception("Invalid parameter with option disassembler. Valid parameters are `standard` or `objdump`")
        elif options[i] == '--input':
            if options[i+1][0] == '-':
                raise Exception("Invalid parameter for option --input")
            opt['input'] = options[i+1]
            i+=1
        elif options[i] == '--flags':
            lst = []
            while i+1 < len(options) and options[i+1][0] != '-':
                lst.append('-' + options[i+1])
                i+=1
            if len(lst) == 0:
                raise Exception("Parameter missing for option --output")
            opt['flags'] = lst
        elif options[i] == '--output':
            opt['output'] = options[i+1]
            i+=1
        elif options[i] == '--setup' or options[i] == '-s':
            opt['setup'] = options[i+1]
            i+=1
        elif options[i] == '--settings':
            s = options[i+1]
            if os.path.exists(f"{const.root}/{s}"):
                if os.path.isdir(f"{const.root}/{s}"):
                    opt['settings'] = [f"{const.root}/{s}/{i}" for i in os.listdir(f"{const.root}/{s}")]
                else:
                    opt['settings'] = [f"{const.root}/{s}"]
            elif os.path.exists(s):
                opt['settings'] = [s]
            else:
                raise Exception("Invalid settings path")
            i += 1
            
        elif options[i] == '--compiler' or options[i] == '-c':
            opt['compiler'] = options[i+1]
            i+=1
        elif options[i] == '-j':
            opt['nbprocess'] = int(options[i+1])
            i+=1

        elif options[i] == '--header' or options[i] == '-H':
            lst = []
            while i+1 < len(options) and options[i+1][0] != '-':
                lst.append(options[i+1])
                i+=1
            if len(lst) == 0:
                raise Exception("Parameter missing for option --header")
            opt['headers'] = lst

        else:
            raise Exception(f"Invalid option {options[i]}")
        i+=1

    return opt



def main(opt):

    if opt['generate']:
        return generation.generate(opt, opt['limit_per_file'])
    elif opt['validate']:
        return validation.validate(opt, opt['limit_per_file'])
    elif opt['analyze']:
        return analyze(opt, opt['compiler'], opt['setup'])



def run():
    argv = sys.argv
    
    options = options_to_dict(argv)
    return main(options)

if __name__ == '__main__':
    run()


import unittest
import sys
import os
sys.path.append(f"{os.path.dirname(__file__)}/../src")
import reader
import const
import numpy


path = f"{os.path.dirname(__file__)}"


class TestReader(unittest.TestCase):
    def __init__(self, methodName: str = "runTest") -> None:
        super().__init__(methodName)

    def test_reader_reference_file(self):
        self.assertEqual(reader.load_json(f"{path}/reader/test1.json"), '{}')

        with self.assertRaises(Exception):
            reader.load_json('.')
        with self.assertRaises(FileNotFoundError):
            reader.load_json(f'{path}/reader/test.json')

        self.assertNotEqual(reader.load_json(f"{path}/reader/test2.json"), "{}")


    def test_keytypes_to_types(self):
        t1 = reader.keytypes_to_types([['float', 'float'], ['int', 'int'], ['double', 'double']])
        self.assertEqual(t1, [['float', 'float'], ['int', 'int'], ['double', 'double']])


        t2 = reader.keytypes_to_types([['arithmetic']])
        self.assertEqual(t2, [[i] for i in const.ARITHMETIC])

        t3 = reader.keytypes_to_types([['arithmetic', 'arithmetic']])
        self.assertEqual(t3, [[i, i] for i in const.ARITHMETIC])

        t4 = reader.keytypes_to_types([['arithmetic', 'arithmetic', 'float']])
        self.assertEqual(t4, [[i, i, 'float'] for i in const.ARITHMETIC])

        t5 = reader.keytypes_to_types([['double', 'arithmetic', 'arithmetic', 'float']])
        self.assertEqual(t5, [['double', i, i, 'float'] for i in const.ARITHMETIC])

        t6 = reader.keytypes_to_types([['double', 'arithmetic', 'arithmetic', 'float'], ['arithmetic', 'float']])
        self.assertEqual(t6, [['double', i, i, 'float'] for i in const.ARITHMETIC] + [[i, 'float'] for i in const.ARITHMETIC])

        t7 = reader.keytypes_to_types([[], ['double', 'arithmetic', 'arithmetic', 'float'], [], ['arithmetic', 'float'], []])
        self.assertEqual(t7, [['double', i, i, 'float'] for i in const.ARITHMETIC] + [[i, 'float'] for i in const.ARITHMETIC])

        t8 = reader.keytypes_to_types([['real', 'arithmetic']])
        for i in t8:
            self.assertTrue(i[0] in const.REAL and i[1] in const.ARITHMETIC)
        self.assertEqual(len(t8), len(const.ARITHMETIC) + len(const.REAL))

        t9 = reader.keytypes_to_types([['real', 'arithmetic', 'arithmetic', 'integer']])
        for i in t9:
            self.assertTrue(i[0] in const.REAL and i[1] in const.ARITHMETIC and i[2] in const.ARITHMETIC and i[3] in const.INTEGER)
        self.assertEqual(len(t9), len(const.ARITHMETIC) + len(const.REAL) + len(const.INTEGER))

        fst = [i[0] for i in t9]
        for i in const.REAL:
            self.assertTrue(i in fst)
        for i in fst:
            self.assertTrue( i in const.REAL )


        self.assertTrue( numpy.array(t1).ndim == 2 )
        self.assertTrue( numpy.array(t2).ndim == 2 )
        self.assertTrue( numpy.array(t3).ndim == 2 ) 
        self.assertTrue( numpy.array(t4).ndim == 2 )
        self.assertTrue( numpy.array(t5).ndim == 2 )
        self.assertTrue( numpy.array(t8).ndim == 2 )
        self.assertTrue( numpy.array(t9).ndim == 2 )

if __name__ == '__main__':
    unittest.main()


from asmat.generation import *
import os
import asmat.reader as reader
import time
import asmat.const as const
from asmat.option import setup


LOG_PATH = f"{const.root}/log.txt"


class AssemblyMismatch(Exception):
    """Assembly mismatch exception

    Args:
        Exception (str, str, str, list, list): Raised when -r option is True.
    """
    def __init__(self, function:str, compiler:str, arch:str, expected:list, actual:list, *args: object) -> None:
        super().__init__(*args)
        self.function = function
        self.compiler = compiler
        self.arch = arch
        self.expected = expected
        self.actual = actual

    def format_instructions(self, instr):
        s = ""
        for i in instr:
            s = s + '\t' + i['instr'] + '\n'
        return s

    def __str__(self) -> str:
        s0 = f'\n-------------------------------------------------\nAssembly mismatch with function : {self.function}\n\n'
        s1 = f'Compiler : {self.compiler}\nArchitecture : {self.arch}\n'
        #s2 = 'Expecting :\n' + self.format_instructions(self.expected) + '\n'
        #s3 = 'Got:\n' + self.format_instructions(self.actual) + '\n'

        return s0 + s1 #+ s2 + s3



def log_unmatched_instruction(function:str, compiler:str, arch:str, types:list, expected:list, actual:list):
    """Format function for the log file.

    Args:
        function (str): Function name.
        compiler (str): Compiler name.
        arch (str): Architecture name.
        types (list): List of parameters types.
        expected (list): List of expected assembly.
        actual (list): List of actual assembly.

    Returns:
        str: Log string.
    """

    s = f'-----------------------------\nFunction : {function}\nCompiler : {compiler}\nArchitecture : {arch}\n'
    s += f'Parameters : {" ".join(types)}\n'
    s += f'\n\n\tExpected\tActual\n\n'
    for i in range(0, max(len(expected), len(actual))):
        if len(expected) <= i:
            s += f'--> ____ \t--> {actual[i]}\n'
        elif len(actual) <= i:
            s += f'--> {expected[i]} \t--> ____\n'
        else:
            if expected[i] != actual[i]:
                s += f'--> {expected[i]}\t--> {actual[i]}\n'
            else:
                s += f'\t{expected[i]}\t{actual[i]}\n'

    return s + '\n\n\n'





def log_undefined_reference(function:str, compiler:str, architecture:str):
    """Format undefined reference error for log file. 

    Args:
        function (str): Function name.
        compiler (str): Compiler name.
        architecture (str): Architecture name.

    Returns:
        str: Log string.
    """

    s = f'-----------------------------\nUndefined reference for function {function}\nCompiler : {compiler}\nArchitecture : {architecture}\n'
    s1 = "\nThis function has not been tested. Try generating reference using option -g\n\n"
    return s + s1



def validate_bis(options, conf={}) -> int:
    """Validation function. Generates assembly for the current library version and compares it with reference assembly.

    Args:
        options (_type_): Dictionary describing user query. The structure of the dictionary is stored in `const.OPTIONS`.
        conf (dict, optional): Configuration with all information about compilation targets. Defaults to {}.

    Raises:
        AssemblyMismatch: Raised when errors occur and raise_exception is True.

    Returns:
        int: 0 if process finished without errors, otherwise -1.
    """

    functions = []
    for k in conf.keys():
        for typ in conf[k]:
            functions.append((k, typ))

    functions_assembly = instructions.get_functions_instructions(options, functions)

    
    validation_set = reader.read_reference_files(options['input'], path=options['output'])

    ret = 0
    errors = 0
    log_txt = ""

    for c in functions_assembly.keys(): # compiler
        for a in functions_assembly[c].keys(): # simd extension
            for f in functions_assembly[c][a].keys(): # function
                if f not in validation_set[c][a].keys():
                    log_txt += log_undefined_reference(f, c, a)
                    if options['verbose']:
                        print(f"**WARNING** - Function {f} not found in references (comp {c}, {a}). Try generating using option -g.")
                    ret = -1
                elif not functions_assembly[c][a][f] == validation_set[c][a][f]:
                    for i in range(0, min(len(functions_assembly[c][a][f]), len(validation_set[c][a][f]))):
                        
                        # Compare only assembly instruction, ignore parameters
                        if options['instruction_comparison']:
                            for j in range(0, min(len(functions_assembly[c][a][f][i]['instr']), len(validation_set[c][a][f][i]['instr']))):
                                i1 = functions_assembly[c][a][f][i]['instr'][j].split(' ')[0]
                                i2 = validation_set[c][a][f][i]['instr'][j].split(' ')[0]
                                if i1 != i2:
                                    ret += 1
                                    errors += 1
                                    if options['exception']:
                                        raise AssemblyMismatch(f, c, a, validation_set[c][a][f][i]['type'], validation_set[c][a][f][i]['instr'], functions_assembly[c][a][f][i]['instr'])
                                    log_txt += log_unmatched_instruction(f, c, a, validation_set[c][a][f][i]['type'], validation_set[c][a][f][i]['instr'], functions_assembly[c][a][f][i]['instr'])
                            
                        elif functions_assembly[c][a][f][i] != validation_set[c][a][f][i]:
                            errors += 1
                            if options['exception']:
                                raise AssemblyMismatch(f, c, a, validation_set[c][a][f][i]['type'], validation_set[c][a][f][i]['instr'], functions_assembly[c][a][f][i]['instr'])
                            log_txt += log_unmatched_instruction(f, c, a, validation_set[c][a][f][i]['type'], validation_set[c][a][f][i]['instr'], functions_assembly[c][a][f][i]['instr'])
                            ret += 1

    # Log file optional
    if options['log'] and log_txt != "":
        if os.path.exists(LOG_PATH):
            file = open(LOG_PATH, 'w')
        else:
            file = open(LOG_PATH, 'x')
        file.write(log_txt)
        file.close()

    if options['verbose']:
        print(log_txt)
        print(f"Process finished : {errors} mismatching function(s) found")

    return ret





def validate(options : dict | setup, max_function_files='inf') -> int:
    """Validation function. Generates assembly for the current library version and compares it with reference assembly.

    Args:
        options (_type_): Dictionary describing user query. The structure of the dictionary is stored in `const.OPTIONS`.
        max_function_files (str, optional): The maximum number of function in a cpp file. A function is one config file (it can generate more than one if there are several\
              parameters configuration). Defaults to 'inf'.

    Returns:
        int: 0 if there is no error, otherwise -1.
    """
    acc_settings_output = 0

    for i in options['settings']:
        const.settings = i
        
        if options['verbose']:
            print(f"Running {i}")

        if type(options) == setup:
            options = options.get_dictionary()
        
        
        t1 = time.time()
        
        conf = reader.read_config_file(options['input'])

        if max_function_files == 'inf' or len(list(conf.keys())) < max_function_files:
            result = validate_bis(options, conf)
        else:
            conf2 = {}
            i = 0
            result = 0
            for k in conf.keys():
                if i == max_function_files:
                    result += validate_bis(options, conf2)
                    conf2 = {}
                    i = 0
                conf2[k] = conf[k]
                i+=1
            result = 1 if result != 0 else 0

        if options['verbose']:
            print("Process duration: " + str(round(time.time()-t1, 2)) + "s")

        acc_settings_output += result

    return not (acc_settings_output == 0)

"""
'absmax': [['float', 'float'], ['double', 'double'], ['std::int64_t', 'std::int64_t'], ['std::int32_t', 'std::int32_t'], 
           ['std::int16_t', 'std::int16_t'], ['std::int8_t', 'std::int8_t'], ['std::uint64_t', 'std::uint64_t'], ['std::uint32_t', 'std::uint32_t'], 
           ['std::uint16_t', 'std::uint16_t'], ['std::uint8_t', 'std::uint8_t'], ['std::uint64_t', 'std::uint64_t'], 
           ['std::uint32_t', 'std::uint32_t'], ['std::uint16_t', 'std::uint16_t'], ['std::uint8_t', 'std::uint8_t']]


{'absmax': [['float', 'float'], ['double', 'double'], ['std::int64_t', 'std::int64_t'], ['std::int32_t', 'std::int32_t'], ['std::int16_t', 'std::int16_t'], 
['std::int8_t', 'std::int8_t'], ['std::uint64_t', 'std::uint64_t'], ['std::uint32_t', 'std::uint32_t'], ['std::uint16_t', 'std::uint16_t'], 
['std::uint8_t', 'std::uint8_t'], ['std::uint64_t', 'std::uint64_t'], ['std::uint32_t', 'std::uint32_t'], ['std::uint16_t', 'std::uint16_t'], 
['std::uint8_t', 'std::uint8_t']]}
"""

if __name__ == '__main__':
    print(validate(input='all', log_file=True, raise_exception=True, keep_tmp=True))


import subprocess
import os
import asmat.const as const
import asmat.settings as settings

TMP_CPP_FILE_NAME = f"{const.root}/tmp/tmp.cpp"
TMP_O_FILE_NAME = f"{const.root}/tmp/tmp.o"
TMP_ASM_FILE_NAME = f"{const.root}/tmp/tmp.s"



def objdump_process(output_path, tmp_o_file=TMP_O_FILE_NAME):
    if not os.path.exists(output_path):
        f = open(output_path, 'x')
    else:
        f = open(output_path, 'w')
    
    with subprocess.Popen(['objdump', '-d', '-j', '.text', '-C', tmp_o_file], stdout=f, stderr=subprocess.PIPE) as p2:
        p2.wait()
        f.close()
        
    if p2.returncode != 0:
        raise Exception("Compilation error : " + p2.stderr.read().decode()) 



def get_assembler(input_path, output_path, compiler='g++', method='objdump', setup:str=None, default_options=False, wait=True, tmp_o_file=TMP_O_FILE_NAME):
    """Compile and disassemble cpp file.

    Args:
        input_path (str): Path of the cpp file.
        output_path (str): Path of the file containing objdump result.
        compiler (str, optional): Compiler used for compilation. Defaults to 'g++'.
        method (str, optional): Defines the disassembling method. Values are `objdump` or `gcc`.
        setup (str, optional): Simd extension options. Defaults to None.
        default_options (bool, optional): If True default options are append for compilation. Defaults to False.

    Raises:
        Exception: If a compilation error occured (mostly because of the options).
    """

    if type(setup) == str:
        setup = [setup]
 
    if method == 'objdump':
        if default_options:
            p1 = subprocess.Popen([compiler, input_path, '-c', '-o', tmp_o_file] + settings.get_flags() + setup, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        else:
            p1 = subprocess.Popen([compiler, input_path, '-c', '-o', tmp_o_file] + setup, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        if wait:
            p1.wait()
            if p1.returncode != 0:
                raise Exception("Compilation error : " + p1.stderr.read().decode())
            objdump_process(output_path)
        else:
            return p1

    else:
        if default_options:
            p1 = subprocess.Popen([compiler, '-S', input_path, '-o', output_path] + settings.get_flags() + setup)
        else:
            p1 = subprocess.Popen([compiler, '-S', input_path, '-o', output_path] + setup)

        if wait:
            p1.wait()
            if p1.returncode != 0:
                raise Exception("Compilation error : " + p1.stderr.read().decode())
        else:
            return p1
            




if __name__ == '__main__':
    pass



import re


def read_compiler_assembler(function_name:str, assembler:str, compiler:str):
    """Get assembly instructions for a given function. Assembler must be generated by `gcc` or `clang` compilers (option -S).

    Args:
        function_name (str): The name of the function we read.
        assembler (str): String generated by compilers.
        compiler (str): The compiler that generated assembly code.

    Returns:
        list: List of assembly instructions.
    """

    if compiler == 'clang':

        x = re.findall(f'func_{function_name}[\S]*?@function[\s\S]*?%bb\.0[\s\S]*?Lfunc_end', assembler)
        y = re.findall(f'%bb.0[\s\S]*retq', x[0])
        y = re.sub('#[^\n]*?\n', '\n', y[0])

        y = y.replace(' ', '').replace('\t', ' ').split('\n')
        y = [i.strip() for i in y]
        y = y[1:-1]

        return y


    else:
        x = re.findall(f'func_{function_name}[\s\S]*?cfi_endproc', assembler)

        y = re.findall(f'endbr[0-9]+[\s\S]*cfi_endproc', x[0])
        y = y[0].replace(' ', '').replace('\t', ' ').split('\n')
        y = [i.strip() for i in y]

        if 'endbr64' in y:
            y.remove('endbr64')
        if '.cfi_endproc' in y:
            y.remove('.cfi_endproc')
        if 'ret' in y:
            y.remove('ret')

        return y


def read_objdump_assembler(function_name:str, assembler:str):
    """Get instructions for a given function. Assembler must be objdump generated string.

    Args:
        function_name (str): The name of the function we read.
        assembler (str): String generated by objdump.

    Raises:
        Exception: The requested function is not found.

    Returns:
        list: List of assembly instructions.
    """

    # extracts each instruction line (full line)
    asm = re.split('\n\n', assembler)
    asm = [i for i in asm if f"func_{function_name}" in i]

    if asm == []:
        raise Exception(f"Function {function_name} not found in assembly")
 
    x = re.findall("[0-9a-f]+:\t[^\n\t]+\t[^\n]+", asm[0])

    # instructions list
    instr = []

    for i in x:
        # extract instructions
        s = i.split('\t')
        s[2] = re.sub(' +', ' ', s[2])
        s = s[2].split(' ')
        
        if s[0] != "endbr64" and s[0] != "endbr32":
            if len(s) > 1 and s[1] != "":
                instr.append(s[0] + ' ' + s[1])
            else:
                instr.append(s[0])
    
    for i in range(len(instr)-1, 0, -1):
        if instr[i] == 'ret':
            break
        else:
            instr.pop()

    return instr



if __name__ == '__main__':
    pass


import os
import json
import asmat.const as const
import random

## This file aims to read json files and extract information from them

def load_json(file_name:str, create_if_not_found=False):
    """Returns json file contents as str

    Args:
        file_name (str): The contents of the file
        create_if_not_found (bool, optional): If True the file is created if it doesn't exist. If False it raises an error if the file doesn't exist. Defaults to False.

    Raises:
        FileNotFoundError: If the file is not found and create_if_not_found is False
        Exception: Other errors

    Returns:
        str: The content of the file
    """

    try:
        f = open(file_name)
        t = f.read()
        if t == '':
            t = '{}'
        f.close()
        return t
    except FileNotFoundError:
        if create_if_not_found:
            f = open(file_name, 'x')
            f.write("{}")
            f.close()
            return "{}"
        else:
            raise FileNotFoundError(f"File {file_name} not found")
    except Exception as e:
        raise Exception(f"Error while loading {file_name} : {e}")



def keytypes_to_types(types:list):
    """Converts key types `arithmetic`, `real`, `unsigned`, `integer` to the corresponding cpp types

    Args:
        types (list): 2D list extract from json config files

    Returns:
        list: 2D list with key types replaced by cpp files
    """

    random.seed = 42

    valid_types = []
    
    for t in types:

        if 'arithmetic' in t or 'integer' in t or 'real' in t or 'unsigned' in t:
            if 'arithmetic' in t:
                for a in const.ARITHMETIC:
                    tmp = [a if i == 'arithmetic' else i for i in t]
                    tmp = [const.REAL[random.randint(0, len(const.REAL)-1)] if i == 'real' else i for i in tmp]
                    tmp = [const.INTEGER[random.randint(0, len(const.INTEGER)-1)] if i == 'integer' else i for i in tmp]
                    tmp = [const.UNSIGNED_INTEGER[random.randint(0, len(const.UNSIGNED_INTEGER)-1)] if i == 'unsigned_integer' else i for i in tmp]
                    tmp = [const.SIGNED[random.randint(0, len(const.SIGNED)-1)] if i == 'signed' else i for i in tmp]
                    tmp = [const.SIGNED_INTEGER[random.randint(0, len(const.SIGNED_INTEGER)-1)] if i == 'signed_integer' else i for i in tmp]
                    valid_types.append(tmp)
            if 'integer' in t:
                for a in const.INTEGER:
                    tmp = [const.ARITHMETIC[random.randint(0, len(const.ARITHMETIC)-1)] if i == 'arithmetic' else i for i in t]
                    tmp = [const.REAL[random.randint(0, len(const.REAL)-1)] if i == 'real' else i for i in tmp]
                    tmp = [a if i == 'integer' else i for i in tmp]
                    tmp = [const.UNSIGNED_INTEGER[random.randint(0, len(const.UNSIGNED_INTEGER)-1)] if i == 'unsigned_integer' else i for i in tmp]
                    tmp = [const.SIGNED[random.randint(0, len(const.SIGNED)-1)] if i == 'signed' else i for i in tmp]
                    tmp = [const.SIGNED_INTEGER[random.randint(0, len(const.SIGNED_INTEGER)-1)] if i == 'signed_integer' else i for i in tmp]
                    valid_types.append(tmp)
            if 'real' in t:
                for a in const.REAL:
                    tmp = [const.ARITHMETIC[random.randint(0, len(const.ARITHMETIC)-1)] if i == 'arithmetic' else i for i in t]
                    tmp = [a if i == 'real' else i for i in tmp]
                    tmp = [const.INTEGER[random.randint(0, len(const.INTEGER)-1)] if i == 'integer' else i for i in tmp]
                    tmp = [const.UNSIGNED_INTEGER[random.randint(0, len(const.UNSIGNED_INTEGER)-1)] if i == 'unsigned_integer' else i for i in tmp]
                    tmp = [const.SIGNED[random.randint(0, len(const.SIGNED)-1)] if i == 'signed' else i for i in tmp]
                    tmp = [const.SIGNED_INTEGER[random.randint(0, len(const.SIGNED_INTEGER)-1)] if i == 'signed_integer' else i for i in tmp]
                    valid_types.append(tmp)
            if 'unsigned_integer' in t:
                for a in const.UNSIGNED_INTEGER:
                    tmp = [const.ARITHMETIC[random.randint(0, len(const.ARITHMETIC)-1)] if i == 'arithmetic' else i for i in t]
                    tmp = [const.REAL[random.randint(0, len(const.REAL)-1)] if i == 'real' else i for i in tmp]
                    tmp = [const.INTEGER[random.randint(0, len(const.INTEGER)-1)] if i == 'integer' else i for i in tmp]
                    tmp = [const.SIGNED[random.randint(0, len(const.SIGNED)-1)] if i == 'signed' else i for i in tmp]
                    tmp = [const.SIGNED_INTEGER[random.randint(0, len(const.SIGNED_INTEGER)-1)] if i == 'signed_integer' else i for i in tmp]
                    tmp = [a if i == 'unsigned_integer' else i for i in tmp]
                    valid_types.append(tmp)
            if 'signed_integer' in t:
                for a in const.SIGNED_INTEGER:
                    tmp = [const.ARITHMETIC[random.randint(0, len(const.ARITHMETIC)-1)] if i == 'arithmetic' else i for i in t]
                    tmp = [const.REAL[random.randint(0, len(const.REAL)-1)] if i == 'real' else i for i in tmp]
                    tmp = [const.INTEGER[random.randint(0, len(const.INTEGER)-1)] if i == 'integer' else i for i in tmp]
                    tmp = [const.UNSIGNED_INTEGER[random.randint(0, len(const.UNSIGNED_INTEGER)-1)] if i == 'unsigned_integer' else i for i in tmp]
                    tmp = [const.SIGNED[random.randint(0, len(const.SIGNED)-1)] if i == 'signed' else i for i in tmp]
                    tmp = [a if i == 'signed_integer' else i for i in tmp]
                    valid_types.append(tmp)
            if 'signed' in t:
                for a in const.SIGNED:
                    tmp = [const.ARITHMETIC[random.randint(0, len(const.ARITHMETIC)-1)] if i == 'arithmetic' else i for i in t]
                    tmp = [const.REAL[random.randint(0, len(const.REAL)-1)] if i == 'real' else i for i in tmp]
                    tmp = [const.INTEGER[random.randint(0, len(const.INTEGER)-1)] if i == 'integer' else i for i in tmp]
                    tmp = [const.UNSIGNED_INTEGER[random.randint(0, len(const.UNSIGNED_INTEGER)-1)] if i == 'unsigned_integer' else i for i in tmp]
                    tmp = [const.SIGNED_INTEGER[random.randint(0, len(const.SIGNED_INTEGER)-1)] if i == 'signed_integer' else i for i in tmp]
                    tmp = [a if i == 'signed' else i for i in tmp]
        else:
            valid_types.append(t)
    return valid_types



def get_groups(entry:str):
    """Returns list of config file if the group is found. Otherwise it returns an empty list.

    Args:
        entry (str): The name of the group.

    Returns:
        list: List of config files.
    """

    settings = load_json(const.settings)
    settings = json.loads(settings)
    g = [i for i in settings['groups'] if i['name'] == entry]
    if len(g) > 0 :
        return g[0]['files']
    else:
        return []


def read_headers():
    settings = load_json(const.settings)
    settings = json.loads(settings)
    return settings['headers']
    



def read_config_file(file_name='all'):
    """Reads config files stored in `config` directory. Files are returned as dictionary.

    Args:
        file_name (str, optional): Files we want to read. If all, it reads all the files in `config` directory. Defaults to 'all'.

    Raises:
        Exception: If the file in argument doesn't exist.

    Returns:
        dict: Dictionary with functions names and parameters types.
    """

    if file_name == 'all':
        d = {}
        for f in os.listdir(f"{const.root}/config"):
            txt = load_json(f"{const.root}/config/{f}")
            function = json.loads(txt)
            d[function['function']] = keytypes_to_types(function['parameters'])
        return d
    else:
        g = get_groups(file_name)
        if g != []:
            file_name = g
        else:
            file_name = [file_name]

        d = {}
        for n in file_name:
            if os.path.exists(f"{const.root}/config/{n}"):
                txt = load_json(f"{const.root}/config/{n}")
                function = json.loads(txt)
                d[function['function']] = keytypes_to_types(function['parameters'])
            elif os.path.exists(n):
                txt = load_json(n)
                function = json.loads(txt)
                d[function['function']] = keytypes_to_types(function['parameters'])
            else:
                raise Exception(f"Configuration file {n} not found.")
        return d
        





def read_reference_files(file_name:str, path="test/asm/ref"):
    """Reads a reference file and returns a dictionary. The file is read for every compilers and architectures

    Args:
        file_name (str): The name of the file to read (not the path).
        path (str, optional): The path of the reference folder. Defaults to "test/asm/ref".

    Returns:
        dict: Dictionary containing assembly program for every compiler and architecture.
    """
    references = {}
    for i in os.listdir(path):
        references[i] = {}
        for j in os.listdir(f"{path}/{i}"):
            references[i][j] = {}
            if file_name == 'all':
                for k in os.listdir(f"{path}/{i}/{j}"):
                    d = json.loads(load_json(f"{path}/{i}/{j}/{k}"))
                    references[i][j][d['function']] = d['asm']
            else:
                g = get_groups(file_name)

                if g != []:
                    names = g
                else:
                    names = [file_name]
                for k in names:
                    try:
                        d = json.loads(load_json(f"{path}/{i}/{j}/{k}"))
                        references[i][j][d['function']] = d['asm']
                    except FileNotFoundError:
                        print(f"File {path}/{i}/{j}/{k} not found")

    return references



if __name__ == '__main__':
    #pass
    read_config_file("abs.json")
    #print(read_reference_files('abs'))


import unittest
from test_utils import ignore_warnings
import sys
import os
sys.path.append(f"{os.path.dirname(__file__)}/../src")
import atp
import files
import validation
import const

path = f"{os.path.dirname(__file__)}"

test_mismatch_1 = """{
    "function": "add",
    "asm": [
        {
            "type": [
                "float",
                "float"
            ],
            "instr": [
                "addps %xmm1,%xmm2"
            ]
        }
    ]
}"""

test_mismatch_2 = """{
    "function": "add",
    "asm": [
        {
            "type": [
                "float",
                "float"
            ],
            "instr": [
                "addpm %xmm1,%xmm0"
            ]
        }
    ]
}"""

test_mismatch_3 = """{
    "function": "max",
    "asm": [
        {
            "type": [
                "float",
                "float"
            ],
            "instr": [
                "maxps %xmm1,%xmm0"
            ]
        },
        {
            "type": [
                "double",
                "double"
            ],
            "instr": [
                "maxpd %xmm1,%xmm2"
            ]
        },
        {
            "type": [
                "int",
                "int"
            ],
            "instr": [
                "maxpd %xmm1,%xmm0"
            ]
        }
    ]
}"""

test_match_1 = """{
    "function": "add",
    "asm": [
        {
            "type": [
                "float",
                "float"
            ],
            "instr": [
                "addps %xmm1,%xmm0"
            ]
        }
    ]
}"""



class TestOptionToDict(unittest.TestCase):

    def __init__(self, methodName: str = "runTest") -> None:
        super().__init__(methodName)

    def test_equal_output(self):
        d = const.OPTIONS.copy()

        d['setup'] = "avx"
        test1 = atp.options_to_dict(["..", "-s", "avx"])
        self.assertEqual(test1, d)

        test2 = atp.options_to_dict(["..", "-s", "avx", "-d", "-l"])
        d['log'] = True
        d['deep'] = True
        self.assertEqual(test2, d)

        test3 = atp.options_to_dict(["..", "-s", "avx", "-d", "-l", '-t', '-g'])
        d['generate'] = True
        d["keep_tmp"] = True
        d['validate'] = False
        self.assertEqual(test3, d)

        test4 = atp.options_to_dict(["..", "-s", "avx", "-d", "-l", '-t', '-g', '--input', 'abs.json'])
        d['input'] = 'abs.json'
        self.assertEqual(test4, d)



    def test_input(self):
        d = const.OPTIONS.copy()
        
        test1 = atp.options_to_dict(["..", '--input', 'abs.json', '-l', '-v'])
        d['log'] = True
        d['verbose'] = True
        d['input'] = 'abs.json'

        self.assertEqual(test1, d)

        test2 = atp.options_to_dict(["..", '-l', '--input', 'abs.json', '-v'])
        d['log'] = True
        d['verbose'] = True
        d['input'] = 'abs.json'

        self.assertEqual(test2, d)

        test3 = atp.options_to_dict(["..", '-l', '-v', '--input', 'abs.json'])
        d['log'] = True
        d['verbose'] = True
        d['input'] = 'abs.json'

        self.assertEqual(test3, d)


    def test_error_invalid_option(self):
        with self.assertRaises(Exception):
            atp.options_to_dict(['..', 's', 's'])
        with self.assertRaises(Exception):
            atp.options_to_dict(['..', ''])
        with self.assertRaises(Exception):
            atp.options_to_dict(['..', '-', 's'])
        with self.assertRaises(Exception):
            atp.options_to_dict(['..', '-', 't', 's', '-'])
        

    def test_invalid_input_file(self):
        with self.assertRaises(Exception):
            atp.options_to_dict(['..', '--input'])
        with self.assertRaises(Exception):
            atp.options_to_dict(['..', '-input', 'abs.json'])
        with self.assertRaises(Exception):
            atp.options_to_dict(['..', 'abs.json', '--input'])
        with self.assertRaises(Exception):
            atp.options_to_dict(['..', '--input', '-t'])


class TestAtp(unittest.TestCase):

    def __init__(self, methodName: str = "runTest") -> None:
        super().__init__(methodName)

    @ignore_warnings
    def test_valid_references(self):
        files.build_reference_directories(f"{path}/atp")
        if os.path.exists(f"{path}/atp/gcc/sse/add.json"):
            file = open(f"{path}/atp/gcc/sse/add.json", 'w')
        else:
            file = open(f"{path}/atp/gcc/sse/add.json", 'x')
        file.write(test_match_1)
        file.close()

        d = const.OPTIONS.copy()
        d['arch'] = 'sse'
        d['validate'] = True
        d['input'] = "add.json"
        d['compiler'] = 'gcc'
        d['ref_path'] = f"{path}/atp"


        self.assertEqual(atp.main(d), 0)

        files.reset(folder=f'{path}/atp')

    """
    def test_unmatched_references(self):
        files.build_reference_directories(f"{const.root}/test/atp")
        if os.path.exists(f"{const.root}/test/atp/gcc/sse/add.json"):
            file = open(f"{const.root}/test/atp/gcc/sse/add.json", 'w')
        else:
            file = open(f"{const.root}/test/atp/gcc/sse/add.json", 'x')
        file.write(test_mismatch_1)
        file.close()
        
        d = const.OPTIONS.copy()
        d['validate'] = True
        d['compiler'] = "gcc"
        d['input'] = 'add.json'
        d['setup'] = 'sse'
        d['ref_path'] = f"{const.root}/test/atp"

        self.assertEqual(atp.main(d), -1)

        if os.path.exists(f"{const.root}/test/atp/gcc/sse/add.json"):
            file = open(f"{const.root}/test/atp/gcc/sse/add.json", 'w')
        else:
            file = open(f"{const.root}/test/atp/gcc/sse/add.json", 'x')
        file.write(test_mismatch_2)
        file.close()

        self.assertEqual(atp.main(d), -1)


        d['arch'] = 'sse3'
        d['input'] = "max.json"
        if os.path.exists(f"{const.root}/test/atp/gcc/sse4.2/max.json"):
            file = open(f"{const.root}/test/atp/gcc/sse4.2/max.json", 'w')
        else:
            file = open(f"{const.root}/test/atp/gcc/sse4.2/max.json", 'x')
        file.write(test_mismatch_3)
        file.close()

        self.assertEqual(atp.main(d), -1)

        const.OPTIONS['exception'] = True
        with self.assertRaises(validation.AssemblyMismatch):
            atp.main(d)
        
        files.reset(folder=f'{const.root}/test/atp')
    """

    @ignore_warnings
    def test_gen_and_valid(self):
        files.build_reference_directories(f"{path}/atp")

        d = const.OPTIONS.copy()
        d['ref_path'] = f"{path}/atp"
        d['deep'] = True
        d['generate'] = True

        atp.main(d)

        d['validate'] = True
        d['generate'] = False

        self.assertEqual(atp.main(d), 0)

        files.reset(folder=f'{path}/atp')




if __name__ == '__main__':
    unittest.main()


import json
import os
import asmat.reader as reader
import asmat.const as const
from asmat.reader import get_groups



def get_setup(entry:str):
    """Returns simd extensions with their corresponding compiler flag. If `entry` is not found in the settings file\
    it is considered as a custom flag.

    Args:
        entry (str): Name of the simd extension to search.

    Returns:
        dict: Dictionary associating entry with its corresponding flag.
    """
    
    settings = reader.load_json(const.settings)
    settings = json.loads(settings)
    if entry == 'all':
        return settings['setup']
    else:
        if entry in settings['setup'].keys():
            return {entry : settings['setup'][entry]}
        else:
            return {entry : entry}
    

def get_flags():
    settings = reader.load_json(const.settings)
    settings = json.loads(settings)
    return settings['flags']


def get_type_wrapper():
    settings = reader.load_json(const.settings)
    settings = json.loads(settings)
    if settings['type_wrapper'].strip() == "":
        return "{}"
    else:
        return settings['type_wrapper']


def get_namespace():
    settings = reader.load_json(const.settings)
    settings = json.loads(settings)
    return settings['function_namespace']


def get_compiler(entry:str):
    """Returns compilers with there corresponding name in the system. If `entry` is found in settings file\
    the corresponding path is returned as the value in the dictionary. Otherwise `entry` is defined as the value and\
    the key is the basename.

    Args:
        entry (str): Name of the compiler.

    Returns:
        dict: The compiler's name associated with the path to execute it.
    """

    settings = reader.load_json(const.settings)
    settings = json.loads(settings)
    if entry == 'all':
        return settings['compilers']
    else:
        if entry in settings['compilers'].keys():
            return {entry : settings['compilers'][entry]}
        else:
            return {os.path.basename(entry) : entry}





def get_target(options:dict):
    """Returns all necessary information about the target. The returned dictionary contains information about:
    - compilers
    - simd extensions
    - functions
    - input
    - type_wrapper
    - output

    Args:
        options (dict): Dictionary describing user query. The structure of the dictionary is stored in `const.OPTIONS`.

    Returns:
        dict: Information about the target.
    """

    d = {}
    if options['flags'] != []:
        if options['compiler'] == None or options['compiler'] == 'all':
            d['compiler'] = get_compiler('all')
        else:
            c = get_compiler(options['compiler'])
            if c == {}:
                d['compiler'] = options['compiler']
            else:
                d['compiler'] = c
        d['setup'] = { 'custom' : options['flags'] }
        
    else:
        if options['compiler'] == None or options['compiler'] == 'all':
            d['compiler'] = get_compiler('all')
        else:
            c = get_compiler(options['compiler'])
            if c == {}:
                d['compiler'] = options['compiler']
            else:
                d['compiler'] = c
        if options['setup'] == None or options['compiler'] == 'all':
            d['setup'] = get_setup('all')
        else:
            d['setup'] = get_setup(options['setup'])

    if input == 'all':
        d['input'] = reader.read_config_file('all')
    else:
        d['input'] = reader.read_config_file(options['input'])
    
    d['twrapper'] = get_type_wrapper()
    d['namespace'] = get_namespace()

    d['output'] = options['output']
    return d
        
            





if __name__ == '__main__':
    print(get_groups('abs'))


import asmat.const as const


class setup:
    """Class that represents options to input. Call method `help()` for details.
    """


    def __init__(self) -> None:
        self.log = False
        self.deep = False
        self.input = 'all'
        self.keep_tmp = False
        self.exception = False
        self.verbose = False
        self.disassembler = "objdump"
        self.instruction_comparison = False
        self.flags = []
        self.setup = None
        self.compiler = None
        self.output = f"{const.root}/ref"
        self.limit_file = 'inf'
        self.nbprocess = 0
        self.headers = []
        self.settings = [f"{const.root}/settings.json"]

    def get_dictionary(self):
        """Returns dictionary with all options for generation, validation and analysis

        Returns:
            dict: Dictionary with user options
        """

        d = {
            "validate" : False,
            "log" : self.log,
            "deep" : self.deep,
            "input" : self.input,
            "keep_tmp" : self.keep_tmp,
            "generate" : False,
            "exception" : self.exception,
            "verbose" : self.verbose,
            "disassembler" : self.disassembler,
            "instruction_comparison" : self.instruction_comparison,
            "flags" : self.flags,
            "setup" : self.setup,
            "compiler" : self.compiler,
            "output" : self.output,
            "limit_per_file" : self.limit_file,
            "nbprocess" : self.nbprocess,
            "headers" : self.headers,
            "settings" : self.settings
        }
        return d
    
    def help(self):
        """Gives details about options
        """
        print("Options:")
        print("\t**log** : Boolean. True to generate log file if an error occurs. Defaults to False.")
        print("\t**deep** : Boolean. True to replace files previously referenced. Defaults to False.")
        print("\t**input** : String. Config files to take as input. Defaults to 'all'.")
        print("\t**keep_tmp** : Boolean. True to keep temporary files. Defaults to False.")
        print("\t**exception** : Boolean. True to raise exception when an error occurs. Defaults to False.")
        print("\t**verbose** : Boolean. True to print information in command line. Defaults to False.")
        print("\t**disassembler** : String. Defines the disassembly mode. Correct values are 'objdump' and 'standard'. Defaults to 'objdump'")
        print("\t**instruction_comparison** : Boolean. True to check instructions and ignore parameters. Defaults to False.")
        print("\t**flags** : string list. List of compilation options without '-'. Defaults to []")
        


if __name__ == '__main__':
    s = setup()
    s.help()


import sqlite3
import datetime

def get_id():
    con = sqlite3.connect("test/asm/src/analyze/data.db")
    cur = con.cursor()
    res = cur.execute(f"SELECT id FROM instructions WHERE edited='Never'")
    res = res.fetchone()
    if res != None:
        return int(res[0])
    else:
        return -1

id = get_id()
setId = False
iname = ""

while True:
    id = get_id()


    if id == -1:
        print("No row selected")
    else:
        con = sqlite3.connect("test/asm/src/analyze/data.db")
        cur = con.cursor()
        res = cur.execute(f"SELECT instr_name, id, description, extension FROM instructions WHERE id = {id}").fetchone()
        if res == None:
            id = -1
            print("Invalid id")
        else:
            iname = res[0]
            print(f"Instruction : {res[0]} (id {res[1]})\nExtension : {res[2]}\n{res[3]}")


    i = input(">>")
    if i == 'exit':
        break

    if 'id' in i:
        id = int(i.split(' ')[1])
        continue
    
    if i == 'pass':
        con = sqlite3.connect("test/asm/src/analyze/data.db")
        cur = con.cursor()
        cur.execute(f"UPDATE instructions SET edited='Pass' WHERE id={id} OR instr_name='{iname}';")
        con.commit()
        con.close()


    if 'set' in i:
        if id == -1:
            print("No data selected")
            continue
        else:
            s = i.split(' ')[1]
            if len(s) != 3 or s[0] != '0' and s[0] != '1' or s[1] != '0' and s[1] != '1' or s[2] != '0' and s[2] != '1':         
                print("Invalid set")
                continue
            con = sqlite3.connect("test/asm/src/analyze/data.db")
            cur = con.cursor()
            cur.execute(f"UPDATE instructions SET ctrl_flow = {s[0]}, arith_logic={s[1]}, data_move={s[2]}, edited='{str(datetime.datetime.now())}' WHERE id={id} OR instr_name='{iname}';")
            con.commit()
            con.close()



from test_extract_assembly import *
from test_atp import *
from test_generation import *
from test_reader import *
import unittest
import warnings

def run_all():
    unittest.main()

def clear_tests():
    os.system("rm -rf test/asm/test/generation/*") # clear files

if __name__ == '__main__':
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        run_all()


import os
if __name__ == "__main__":
    import sys
    sys.path.append("/home/erwan/eve/test/asm/src")
import re

from asmat.analysis.parse_html import is_extension_instruction, instruction_categories
import asmat.const as const
import matplotlib.pyplot as plt



FUNCTION_PAGE_STYLE = """
<style>
    span:hover { cursor: default; opacity: 0.5; }
    body {margin: auto;max-width: 60%;padding-top: 1%;}
    h3 {margin: 4px;}
    .row {display: flex;}
    .column {flex: 50%;}
    pre code {background-color: #eee;border: 1px solid #999;display: block;padding: 20px;}
    td, th {border: 1px solid #DDD;padding: 4px;text-align: left;}
    tr:nth-child(even){background-color: #f2f2f2;}
</style><html><body><div class="row"><div class="column">"""

DOCUMENT_STYLE = """
<style>
    td, th {border: 1px solid #DDD;padding: 4px;text-align: left;}
    tr:hover {background-color: #D6EEEE;}
    table {border-collapse: collapse;width: 100%;text-align: center;margin-top: 10%;}
    body {margin: auto;max-width: 60%;padding-top: 1%;}
    #head {background-color: gainsboro;}
</style>
"""





def gen_pie_chart(path, proportion:list):
    fig, ax = plt.subplots()
    ax.pie(proportion, labels=['Control flow', 'Arithmetic/Logic', 'Data movement', 'Undefined'], autopct='%1.1f%%', shadow=True, startangle=90)
    plt.savefig(f'{const.root}/output/pages/{path}.jpg')
    plt.close(fig)


def data_assembly(instructions):
    colors = []
    ext = []
    nbExt = 0
    mem = 0

    cat0 = 0
    cat1 = 0
    cat2 = 0
    cat3 = 0

    for i in instructions:
        e = is_extension_instruction(i.split(' ')[0].upper())

        cat = instruction_categories(i.split(' ')[0].upper())
        cat0 += cat[0]
        cat1 += cat[1]
        cat2 += cat[2]
        cat3 += cat[3]

        if cat[0] == 1:
            colors.append((i, "Control flow", "blue"))
        elif cat[1] == 1:
            colors.append((i, "Arithmetic/Logic", " #e67e22"))
        elif cat[2] == 1:
            colors.append((i, "Data movement", "green"))
        else:
            colors.append((i, "Undefined", "red"))

        if e != None:
            if e not in ext:
                ext.append(e)
            nbExt += 1

        if re.match(".*0x[0-9a-f]+\(.*\).*", i):
            mem += 1

    if ext == []:
        ext = ['No extension used']

    result = {}
    result['cat0'] = cat0
    result['cat1'] = cat1
    result['cat2'] = cat2
    result['cat3'] = cat3
    result['ext'] = ext
    result['nbExt'] = nbExt
    result['mem'] = mem
    result['instr'] = colors

    return result


def generate_function_page(page_name:str, function:str, dataInstr:dict):

    if not os.path.exists(f"{const.root}/output/pages/{page_name}.html"):
        f = open(f"{const.root}/output/pages/{page_name}.html", 'x')
    else:
        f = open(f"{const.root}/output/pages/{page_name}.html", 'w')

    doc = FUNCTION_PAGE_STYLE
    doc += f"<h1>{function}</h1>"                        # Function name
    doc += f"<h3>Instructions: {len(dataInstr['instr'])}</h3>" # Number of instructions
    doc += f"<h3>Extension(s): {', '.join(dataInstr['ext'])}</h3>"               # List of extensions
    doc += f"<h3>Memory access: {dataInstr['mem']}</h3>"                # Number of memory access

    gen_pie_chart(page_name, [dataInstr['cat0'], dataInstr['cat1'], dataInstr['cat2'], dataInstr['cat3']])

    doc += f'<table style="border-collapse: collapse;border: 2px solid #eee; margin-top: 30px;max-width: 80%;min-width: 50%;">\
    <tr><th>Data movement</th><td>{dataInstr["cat2"]}</td></tr><tr><th>Logic/Arithmetic</th><td>{dataInstr["cat1"]}</td></tr><tr><th>Control flow</th><td>{dataInstr["cat0"]}</td>\
        </tr><tr><th>Undefined</th><td>{dataInstr["cat3"]}</td></tr></table></div>'
    doc += f'<div class="column"><img src="{page_name}.jpg" alt="" style="max-width: 100%;margin-top: 50px;"></div></div>'

    doc += f'<div><h2>Instructions</h2><pre><code style="max-height: 300px;overflow: scroll;">'

    for i in dataInstr['instr']:
        doc += f'<span style="background-color: {i[2]};" title="{i[1]}"> </span> {i[0]}\n'
    
    doc += "</code></pre></div></body></html>"

    f.write(doc)
    f.close()







def generate_index(functions):

    if not os.path.exists(f"{const.root}/output/index.html"):
        f = open(f"{const.root}/output/index.html", 'x')
    else:
        f = open(f"{const.root}/output/index.html", 'w')
    
    doc = "<html><body><table><h1>Assembly analysis</h1><tr id=\"head\"><th>Function</th>\
        <th>Instructions</th><th>Extensions</th></tr>"

    for i in functions:
        function_name = f"{i[0]}({', '.join(i[1])})"
        page_name = ''.join( map(lambda i : i if i.isalnum() else '_' , function_name) )

        dataInstr = data_assembly(i[2])

        doc += f'<tr><td><a href="pages/{page_name}.html">{function_name.replace("<", "&lt;").replace(">", "&gt;")}</a></td><td>{len(i[2])}</td><td>{", ".join(dataInstr["ext"])}</td></tr>'
        generate_function_page(page_name, function_name, dataInstr)

    doc += "</table></body></html>"

    f.write(DOCUMENT_STYLE + doc)
    f.close()





if __name__ == '__main__':
    print(''.join( map(lambda i : i if i.isalnum() else '_' , "res_defined_array(int*)")) )


import json
import asmat.instructions as instructions
import asmat.const as const
import asmat.reader as reader
import os
import time
from asmat.option import setup



def save_json(destination_path:str, text:str):
    """Save string in a file.

    Args:
        destination_path (str): Path to save file.
        text (str): File content.

    Raises:
        Exception: Raised when error occured while writing the file.
    """
    if not os.path.exists(destination_path):
        try:
            f = open(destination_path, 'x')
            f.write(text)
            f.close()
        except Exception as e:
            raise Exception(f"Error while writing {destination_path} : {e}")
    else:
        try:
            f = open(destination_path, 'w')
            f.write(text)
            f.close()
        except Exception as e:
            raise Exception(f"Error while writing {destination_path} : {e}")





def generate_bis(options:dict, conf:dict) -> int:
    """Generate and store assembly code.

    Args:
        options (dict): Dictionary describing user query. The structure of the dictionary is stored in `const.OPTIONS`.
        conf (dict): Configuration with all information about compilation targets.

    Returns:
        int: 0
    """

    output_directory = options['output']
    deep = options['deep']
    verbose = options['verbose']

    if output_directory == None:
        output_directory = f"{const.ref_path}"

    functions = []
    for k in conf.keys():
        for typ in conf[k]:
            functions.append((k, typ))

    functions_assembly = instructions.get_functions_instructions(options, functions)

    replaced_files = 0

    for comp in functions_assembly.keys():
        for arch in functions_assembly[comp].keys():
            for f in functions_assembly[comp][arch].keys():
                try:
                    os.makedirs(f"{output_directory}/{comp}/{arch}")
                except FileExistsError:
                    pass

                if not os.path.exists(f"{output_directory}/{comp}/{arch}/{f}.json"):
                    file = open(f"{output_directory}/{comp}/{arch}/{f}.json", 'x')
                    file.close()
                    dict_json = { "function" : f, "asm" : functions_assembly[comp][arch][f] }
                    update_json = json.dumps(dict_json, indent=4)
                    save_json(f"{output_directory}/{comp}/{arch}/{f}.json", update_json)
                    replaced_files += 1

                elif reader.load_json(f"{output_directory}/{comp}/{arch}/{f}.json") == '' or reader.load_json(f"{output_directory}/{comp}/{arch}/{f}.json") == '{}' or deep:
                    dict_json = { "function" : f, "asm" : functions_assembly[comp][arch][f] }
                    
                    update_json = json.dumps(dict_json, indent=4)
                    save_json(f"{output_directory}/{comp}/{arch}/{f}.json", update_json)
                    replaced_files += 1

    if verbose:
        print(f"Operation finished : {replaced_files} functions saved")

    return 0
    




def generate(options: dict | setup, max_function_files='inf') -> int:
    """Generate and store assembly code.

    Args:
        options (dict): Dictionary describing user query. The structure of the dictionary is stored in `const.OPTIONS`
        max_function_files (str, optional): The maximum number of function in a cpp file. A function is one config file (it can generate more than one if there are several\
              parameters configuration). Defaults to 'inf'.

    Returns:
        int: 0 if there is no error, otherwise -1.
    """

    acc_settings_output = 0

    for i in options['settings']:
        const.settings = i
        if options['verbose']:
            print(f"Running {i}")

        if type(options) == setup:
            options = options.get_dictionary()

        t1 = time.time()


        conf = reader.read_config_file(options['input'])

        if max_function_files == 'inf' or len(list(conf.keys())) < max_function_files:
            result = generate_bis(options, conf)
        else:
            conf2 = {}
            i = 0
            result = 0
            for k in conf.keys():
                if i == max_function_files:
                    result += generate_bis(options, conf2)
                    conf2 = {}
                    i = 0
                conf2[k] = conf[k]
                i+=1
            result = -1 if result != 0 else 0

        if options['verbose']:
            print("Process duration: " + str(round(time.time()-t1, 2)) + "s")

        acc_settings_output += result

    return not (acc_settings_output == 0)


if __name__ == '__main__':
    pass


import os

root = os.getcwd()

settings = f"{root}/settings.json"

VERSION = "1.1.1"

ARCH = {
    'sse' : '-msse', 
    'sse2' : '-msse2',
    'sse3' : '-msse3',
    'sse4.2' : '-msse4.2', 
    'ssse3' : '-mssse3',
    'avx' : '-mavx',
    'avx2' : '-mavx2',
    'avx512' : '-march=skylake-avx512'}


COMPILER = {
    'clang' : 'clang',
    'gcc' : 'g++'
}


OPTIONS = {
    "validate" : True,
    "analyze" : False,
    "generate" : False,
    "log" : False,
    "deep" : False,
    "input" : 'all',
    "keep_tmp" : False,
    "exception" : False,
    "verbose" : False,
    "disassembler" : "objdump",
    "instruction_comparison" : False,
    "flags" : [],
    "setup" : None,
    "compiler" : None,
    "output" : f"{root}/ref",
    "limit_per_file" : 'inf',
    "nbprocess" : 0,
    "headers" : [],
    "settings" : [f"{root}/settings.json"]
}


REAL = ['float', 'double']
SIGNED_INTEGER = ['std::int64_t', 'std::int32_t', 'std::int16_t', 'std::int8_t']
UNSIGNED_INTEGER = ["std::uint64_t" , "std::uint32_t" , "std::uint16_t" , "std::uint8_t"]
INTEGER = SIGNED_INTEGER + UNSIGNED_INTEGER
SIGNED = SIGNED_INTEGER + REAL

ARITHMETIC = REAL + INTEGER + UNSIGNED_INTEGER


import unittest
import os
import sys
from test_utils import ignore_warnings
sys.path.append(f"{os.path.dirname(__file__)}/../src")
import generation
import files
import settings
import const

path = f"{os.path.dirname(__file__)}"



class TestUpdateGeneration(unittest.TestCase):

    def __init__(self, methodName: str = "runTest") -> None:
        super().__init__(methodName)

    @ignore_warnings
    def test_output_files(self):

        os.system(f"rm -rf {path}/generation/*")
        
        d = const.OPTIONS.copy()
        d['output'] = f"{path}/generation/"
        generation.generate(d)

        sett = settings.get_target(d)
        for c in sett['compiler'].keys():
            for a in sett['setup'].keys():
                for f in os.listdir(f"{const.root}/config"):
                    self.assertTrue( os.path.exists( f"{path}/generation/{c}/{a}/{f}" ) )
        
        files.reset(f"{path}/generation")
                                        
                                        

if __name__ == '__main__':
    unittest.main()


import os
if __name__ == "__main__":
    import sys
    sys.path.append("/home/erwan/eve/test/asm/src")

import  asmat.analysis.analysis_output as analysis_output

import asmat.instructions as instructions
import asmat.const as const
import asmat.reader as reader
from asmat.option import setup



def analyze(options:dict | setup, compiler:str=None, cpu_ext:str=None):
    """Analyzes assembly code of c++ functions. Gives a report in html format (in `/output` directory).

    Args:
        options (dict | setup): Setup object or dictionary containing user options.
        compiler (str, optional): The compiler to analyze. Defaults to None.
        cpu_ext (str, optional): The cpu extension to analyze. Defaults to None.

    Raises:
        Exception: If compiler or cpu_ext are not valid.
    """

    if compiler == None or cpu_ext == None:
        raise Exception("Missing compiler and cpu options for analysis.")

    if type(options) == setup:
        options = options.get_dictionary()

    output_directory = options['output']
    conf = reader.read_config_file(options['input'])

    if output_directory == None:
        output_directory = f"{const.ref_path}"

    functions = []
    for k in conf.keys():
        for typ in conf[k]:
            functions.append((k, typ))

    instr = instructions.get_functions_instructions(options, functions)[compiler][cpu_ext]

    index = []

    functions = sorted( list(instr.keys()) )
    for i in functions:
        for j in instr[i]:
            index.append((i, j['type'], j['instr']))

    if not os.path.exists(f"{const.root}/output"):
        os.mkdir(f"{const.root}/output")
    if not os.path.exists(f"{const.root}/output/pages"):
        os.mkdir(f"{const.root}/output/pages")
    
    analysis_output.generate_index(index)

    return 0




if __name__ == '__main__':
    opt = const.OPTIONS.copy()
    opt['input'] = 'all'
    opt['compiler'] = 'g++'
    opt['setup'] = 'avx'
    analyze(opt, 'g++', 'avx')




import setuptools

with open("README.md", "r", encoding = "utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
  name = 'asmat',         # How you named your package folder (MyLib)
  packages=["asmat", "asmat.analysis"],   # Chose the same as "name"
  py_modules=["asmat", "asmat.analysis"],
  include_package_data=True,
  package_dir={"":"src"},
  package_data={"": ["*.db"]},
  version = "1.1.1",      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Assembly Testing and Analysis',   # Give a short description about your library
  long_description=long_description,
  long_description_content_type="text/markdown",
  author = 'Erwan LEMATTRE',                   # Type in your name
  author_email = 'erwanlem4@gmail.com',      # Type in your E-Mail
  url = 'https://github.com/erwanlem/atp',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/erwanlem/atp/archive/refs/tags/v1.0.tar.gz', 
  keywords = ['ASSEMBLER', 'TEST', 'ANALYSIS'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
          'bs4',
          'matplotlib',
          'pillow'
      ],
  classifiers=[
    "Programming Language :: Python :: 3",
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    "Operating System :: OS Independent",
  ],
  python_requires= ">=3.6"
)


import os
import asmat.assembly_parser as assembly_parser
import asmat.const as const
import asmat.settings as settings
import asmat.reader as reader
from asmat.compiler import TMP_ASM_FILE_NAME, TMP_CPP_FILE_NAME, TMP_O_FILE_NAME, get_assembler, objdump_process


FUNCTION_ID = 0


def get_function_id():
    """Returns a uniq id

    Returns:
        int: ID
    """

    global FUNCTION_ID
    FUNCTION_ID = FUNCTION_ID + 1
    return FUNCTION_ID-1




def function_extended_name(f, p):
    """Returns function extended name with function name and parameters used to identify the function.

    Args:
        f (str): Function name
        p (list): Parameters list

    Returns:
        str: Function extended name
    """

    return f + '_' + ''.join(map(str, p))





def generate_function(funcName:str, parameters:list, wrapper="{}", namespace="", functionId='auto'):
    """Generates cpp function code. The function id generated with a uniq id in its name.

    Args:
        funcName (str): Name of the function to call.
        parameters (list): List of the types of the variables used for the call.
        functionId (str, optional): Defines a custom id for the function. If auto the id is generated automatically. Defaults to 'auto'.

    Returns:
        str: Code as str.
    """

    # Generation of the parameters
    formal_param = ""
    real_param = ""
    func_id = funcName + '_' + str(get_function_id()) if functionId == 'auto' else funcName + '_' + str(functionId)

    for i in range(0, len(parameters)):
        if i == len(parameters)-1:
            formal_param += wrapper.format(parameters[i]) + f" a{i}" 
            real_param += f"a{i}"
        else:
            formal_param += wrapper.format(parameters[i]) + f" a{i}, "
            real_param += f"a{i}, "

    # Code of the temporary function
    code = f"auto func_{func_id}(" + formal_param + """) {
    return """ + namespace + funcName + "(" + real_param + """);
}\n"""
    

    return (func_id, code)



def clear_tmp():
    """
    Removes all temporary files
    """

    if os.path.exists(TMP_CPP_FILE_NAME):
        os.system(f"rm {TMP_CPP_FILE_NAME}")
    if os.path.exists(TMP_O_FILE_NAME):
        os.system(f"rm {TMP_O_FILE_NAME}")
    if os.path.exists(TMP_ASM_FILE_NAME):
        os.system(f"rm {TMP_ASM_FILE_NAME}")






def extract_instructions(functionName:str, parameters:str, func_code_name:dict, asm:str, method='objdump', compiler:str='gcc'):
    """Extracts assembly instructions from objdump files.

    Args:
        functionName (str): Name of the function to extract.
        parameters (str): Parameters types of the function.
        func_code_name (dict): Dictionary which associates extended names to "code names". 
        asm (str): Assembly str generated by objdump.
        method (str, optional): Defines the disassembling method. Values are `objdump` or `gcc`.
        compiler (str, optional): The compiler used. Only useful if method=standard.

    Raises:
        Exception: If extended name is not found is func_code_name.
        Exception: If the function is not found in the assembly file.

    Returns:
        list: List with all assembly instructions.
    """
    
    new_name = function_extended_name(functionName, parameters)
    if new_name not in func_code_name.keys():
        raise Exception("ERROR : Invalid function identifiant")

    function_name = func_code_name[new_name]

    if method == 'objdump':
        return assembly_parser.read_objdump_assembler(function_name, asm)
    else:
        return assembly_parser.read_compiler_assembler(function_name, asm, compiler=compiler)




def get_functions_instructions(options, functions : list):
    """Returns assembly instructions for each architectures, compilers and functions. The result is stored in a dictionary DICT[COMPILER][ARCHITECTURE][FUNCTION]['type' or 'instr']

    Args:
        options (_type_): Options describing user query
        functions (list): List of tuples (function_name, parameters list) for each functions we want assembly.

    Returns:
        Dict: Dictionary with assembly programs.
    """

    method = options['disassembler']

    clear_tmp()

    headers = ""
    if options['headers'] == []:
        for i in reader.read_headers():
            headers += f"#include {i}\n"
    else:
        for i in options['headers']:
            headers += f'#include {i}\n'

    full_code = headers
    functions_names = {}
    res_dict = {}
    
    
    target = settings.get_target(options)

    # Generating .cpp code
    for f, p in functions:
        new_name, code = generate_function(f, p, wrapper=target['twrapper'], namespace=target['namespace'])
        full_code += code
        functions_names[function_extended_name(f, p)] = new_name

    if not os.path.isdir(f"{const.root}/tmp"):
        os.mkdir(f"{const.root}/tmp")

    # Write code in tmp.cpp
    tmp = open(TMP_CPP_FILE_NAME, 'x')
    tmp.write(full_code)
    tmp.close()

    

    compilers = target['compiler'].keys()
    architectures = target['setup'].keys()

    queue = []
    for comp in compilers:
        for a in architectures:
            queue.append((comp, a))

    maxproc = options['nbprocess'] if options['nbprocess'] > 0 else os.cpu_count()

    while queue != []:
        files = {}
        file_id = 0

        if options['verbose']:
            print(f'Generating assembly...', end='\r')

        # Run compilations
        for i in range(0, min(maxproc, len(queue))):
            comp, a = queue.pop()

            if comp not in res_dict.keys():
                res_dict[comp] = {}
            if a not in res_dict[comp].keys():
                res_dict[comp][a] = {}
            
            if method == 'objdump':
                p = get_assembler(TMP_CPP_FILE_NAME, f"{const.root}/tmp/tmp{file_id}.s", compiler=comp, method=method,\
                                    setup=target['setup'][a], default_options=options['flags'] == [], wait=False, tmp_o_file=f"{const.root}/tmp/tmp{file_id}.o")
                files[p] = (f"{const.root}/tmp/tmp{file_id}", comp, a)
                file_id += 1
            else:
                p = get_assembler(TMP_CPP_FILE_NAME, f"{const.root}/tmp/tmp{file_id}.s", compiler=comp, method=method, setup=target['setup'][a], default_options=options['flags'] == [], wait=False)
                files[p] = (f"{const.root}/tmp/tmp{file_id}", comp, a)
                file_id += 1

        # Wait for subprocess
        for i, j in files.items():
            i.wait()
            if method == 'objdump':
                if i.returncode != 0:
                    raise Exception("Compilation error : " + i.stderr.read().decode())
                objdump_process(j[0] + '.s', tmp_o_file=j[0] + '.o')

            file_asm = open(j[0] + '.s')
            asm = file_asm.read()
            file_asm.close()

            for f, p in functions:
                if f not in res_dict[j[1]][j[2]].keys():
                    res_dict[j[1]][j[2]][f] = [{"type" : p, "instr" : extract_instructions(f, p, functions_names, asm, method=method, compiler=j[1])}]
                else:
                    res_dict[j[1]][j[2]][f].append({"type" : p, "instr" : extract_instructions(f, p, functions_names, asm, method=method, compiler=j[1])})




    if options['verbose']:
        print(f'Assembly generated successfully')

    # Temporary files 
    if not options['keep_tmp']:
        for i in os.listdir(f"{const.root}/tmp"):
            os.remove(f"{const.root}/tmp/{i}")
        os.rmdir(f"{const.root}/tmp")
        clear_tmp()

    return res_dict


if __name__ == '__main__':
    c = const.OPTIONS.copy()
    c['input'] = 'abs.json'

    get_functions_instructions(c, [("abs", [],)])


import urllib.request
from bs4 import BeautifulSoup
import os
import sqlite3



def load():
    if not os.path.exists("test/asm/src/analyze/source_html.txt"): 
        html = urllib.request.urlopen("https://shell-storm.org/x86doc/index.html").read()
        f = open("test/asm/src/analyze/source_html.txt", "x")
        f.write(str(html))
        f.close()
    else:
        html = open("test/asm/src/analyze/source_html.txt", "r").read()


    parser_html = BeautifulSoup(html, features="html.parser")

    assembly_instructions = []

    for i in parser_html.findAll('tr'):
        parser_html2 = BeautifulSoup(str(i), features="html.parser")
        td = parser_html2.findAll('td')

        if len(td) >= 4:
            l = [td[0].a.get_text(), td[1].get_text(), td[2].get_text(), td[3].get_text()]
        
            assembly_instructions.append(l)

    con = sqlite3.connect(f"{os.path.dirname(__file__)}/data.db")
    cur = con.cursor()
    res = cur.execute("DELETE FROM instructions")
    assert res.fetchone() == None

    cur.execute("UPDATE sqlite_sequence SET seq = 0 WHERE name='instructions'")
    con.commit()

    cur.executemany("INSERT INTO instructions (instr_name, op_code, extension, description, instr_set) VALUES (?, ?, ?, ?, 'x86')", assembly_instructions)
    con.commit()


    res = cur.execute("SELECT DISTINCT instr_name FROM instructions WHERE extension != '' ORDER BY length(instr_name) DESC")
    print(len(res.fetchall()))

    res = cur.execute("SELECT DISTINCT instr_name FROM instructions ORDER BY length(instr_name) DESC")
    print(len(res.fetchall()))


    cur.close()
    con.close()


def is_extension_instruction(instr):
    con = sqlite3.connect(f"{os.path.dirname(__file__)}/data.db")
    cur = con.cursor()
    res = cur.execute("SELECT instr_name, extension FROM instructions WHERE extension != '' ORDER BY length(instr_name) DESC")

    for i in res:
        if i[0] == instr:
            return i[1]
    return None


def instruction_categories(instr):
    con = sqlite3.connect(f"{os.path.dirname(__file__)}/data.db")
    cur = con.cursor()
    res = cur.execute(f"SELECT ctrl_flow, arith_logic, data_move FROM instructions WHERE instr_name='{instr}'")

    res = res.fetchone()
    if res is None:
        return (0, 0, 0, 1)
    if res[0] + res[1] + res[2] == 0:
        return (0, 0, 0, 1)
    else:
        return (res[0], res[1], res[2], 0)


if __name__ == '__main__':
    #load()
    #print(is_extension_instruction("movaps".upper()))
    print(instruction_categories("AAA"))
    pass




import unittest
import sys
from test_utils import ignore_warnings
import os
sys.path.append(f"{os.path.dirname(__file__)}/../src")
from instructions import *
import const

path = f"{os.path.dirname(__file__)}"

class TestGenerateFunction(unittest.TestCase):

    def __init__(self, methodName: str = "runTest") -> None:
        super().__init__(methodName)
        clear_tmp()
    

    def test_valid_file_output(self):
        clear_tmp()

        c1 = generate_function("test1", [], functionId=0)
        t1 = ("test1_0", """auto func_test1_0() {
    return eve::test1();
}
""")

        c2 = generate_function("test2", ['eve::wide<float>'], functionId=1)
        t2 = ("test2_1", """auto func_test2_1(eve::wide<float> a0) {
    return eve::test2(a0);
}
""")
        c3 = generate_function("test3", ['eve::wide<float>', 'eve::wide<int>', 'eve::wide<double>'], functionId=2)
        t3 = ("test3_2", """auto func_test3_2(eve::wide<float> a0, eve::wide<int> a1, eve::wide<double> a2) {
    return eve::test3(a0, a1, a2);
}
""")

        c4 = generate_function("test4", ['eve::wide<int>', 'eve::wide<int>'], functionId=3)
        t4 = ("test4_3", """auto func_test4_3(eve::wide<int> a0, eve::wide<int> a1) {
    return eve::test4(a0, a1);
}
""")
        
        c5 = generate_function("", [], functionId=4)
        t5 = ("_4", """auto func__4() {
    return eve::();
}
""")
        self.assertEqual(c1, t1)
        self.assertEqual(c2, t2)
        self.assertEqual(c3, t3)
        self.assertEqual(c4, t4)
        self.assertEqual(c5, t5)


class TestExtractInstructions(unittest.TestCase):

    def __init__(self, methodName: str = "runTest") -> None:
        super().__init__(methodName)
        self.asm = """00000000000000d0 <func_dist_8(eve::avx512_abi_v0::wide<float, eve::fixed<16l> >, eve::avx512_abi_v0::wide<float, eve::fixed<16l> >)>:
  d0:	f3 0f 1e fa          	endbr64 
  d4:	4c 8d 54 24 08       	lea    0x8(%rsp),%r10
  d9:	48 83 e4 c0          	and    $0xffffffffffffffc0,%rsp
  dd:	41 ff 72 f8          	push   -0x8(%r10)
  e1:	55                   	push   %rbp
  e2:	48 89 e5             	mov    %rsp,%rbp
  e5:	41 52                	push   %r10
  e7:	48 83 ec 28          	sub    $0x28,%rsp
  eb:	e8 30 ff ff ff       	call   20 <eve::avx512_abi_v0::wide<float, eve::fixed<16l> > eve::detail::dist_<eve::avx512_abi_v0::wide<float, eve::fixed<16l> >, eve::options<rbr::settings<rbr::option<eve::detail::condition_key_t, eve::ignore_none_> > > >(eve::detail::adl_helper_t const&, eve::cpu_ const&, eve::options<rbr::settings<rbr::option<eve::detail::condition_key_t, eve::ignore_none_> > > const&, eve::avx512_abi_v0::wide<float, eve::fixed<16l> >, eve::avx512_abi_v0::wide<float, eve::fixed<16l> >) [clone .isra.0]>
  f0:	4c 8b 55 f8          	mov    -0x8(%rbp),%r10
  f4:	c9                   	leave  
  f5:	49 8d 62 f8          	lea    -0x8(%r10),%rsp
  f9:	c3                   	ret    
  fa:	66 0f 1f 44 00 00    	nopw   0x0(%rax,%rax,1)

    0000000000000130 <func_add_10(eve::avx512_abi_v0::wide<float, eve::fixed<16l> >, eve::avx512_abi_v0::wide<float, eve::fixed<16l> >)>:
 130:	f3 0f 1e fa          	endbr64 
 134:	62 f1 74 48 58 c0    	vaddps %zmm0,%zmm1,%zmm0
 13a:	c3                   	ret    
 13b:	0f 1f 44 00 00       	nopl   0x0(%rax,%rax,1)
"""

    @ignore_warnings
    def test_valid_call(self):        
        t1 = extract_instructions('dist', ["float", "float"], {function_extended_name('dist', ["float", "float"]) : "dist_8"}, self.asm)
        t2 = extract_instructions('add', ["signed char", "signed char"], {function_extended_name('add', ["signed char", "signed char"]) : "add_10"}, self.asm)

        self.assertTrue( t1 == ['lea 0x8(%rsp),%r10', 'and $0xffffffffffffffc0,%rsp', 'push -0x8(%r10)', 'push %rbp', 'mov %rsp,%rbp', 'push %r10', 'sub $0x28,%rsp', 'call 20', 'mov -0x8(%rbp),%r10', 'leave', 'lea -0x8(%r10),%rsp'] )
        self.assertTrue( t2 == ['vaddps %zmm0,%zmm1,%zmm0'] )

        with self.assertRaises(Exception):
            extract_instructions('add', ["signed char"], {function_extended_name('add', ["signed char", "signed char"]) : "add_10"}, self.asm)
        with self.assertRaises(Exception):
            extract_instructions('add_10', ["signed char", "signed char"], {function_extended_name('add', ["signed char", "signed char"]) : "add_10"}, self.asm)
        with self.assertRaises(Exception):
            extract_instructions('add', ["signed char", "signed char", "signed char"], {function_extended_name('add', ["signed char", "signed char"]) : "add_10"}, self.asm)
        
        
    @ignore_warnings
    def test_func_not_found_exception(self):
        with self.assertRaises(Exception):
            extract_instructions('00', self.asm)



class TestGetFunctionInstructions(unittest.TestCase):
    def __init__(self, methodName: str = "runTest") -> None:
        super().__init__(methodName)
        self.opt = const.OPTIONS.copy()
        self.opt['compiler'] = "gcc"
        self.opt['setup'] = "sse"
        self.opt['input'] = "all"
        self.opt["output"] = f"{path}/output"


    @ignore_warnings
    def test_valid_entry(self):
        
        t1 = get_functions_instructions(self.opt, [('add', ['int', 'int']), ('abs', ['float'])])

        self.assertEqual(list(t1.keys()), ['gcc'])
        self.assertEqual(list(t1['gcc'].keys()), ['sse'])
        self.assertEqual(list(t1['gcc']['sse'].keys()), ['add', 'abs'])

    @ignore_warnings
    def test_limit_case(self):
        t1 = get_functions_instructions(self.opt, [])

        self.assertEqual(list(t1.keys()), ['gcc'])
        self.assertEqual(list(t1['gcc'].keys()), ['sse'])
        
        for i in t1.keys():
            for j in t1[i].keys():
                self.assertEqual(t1[i][j], {})


if __name__ == '__main__':
    unittest.main()


import os
import asmat.const as const


default_settings = """{
    "compilers":
        {
            // "gcc": "g++" // Example
        },
    "setup":
        {
            // "sse": "-msse" // Example
        }, 
    "groups": [
        /*
            Example
        {
            "name": "myGroup",
            "files": [
                "filename.json"
            ]
        }
        */
    ],
    "headers": [
        // C++ headers
    ],
    "flags": [
        // "-std=c++20" // Example
    ],
    "type_wrapper": ""
}"""




def build_dependencies(replace=False):
    """Build setting and configuration files.

    Args:
        replace (bool, optional): True to replace files if they already exist. Defaults to False.
    """
    if not os.path.exists(f"{const.root}/ref"):
        os.mkdir(f"{const.root}/ref")
    elif replace:
        os.remove(f"{const.root}/ref")
        os.mkdir(f"{const.root}/ref")

    if not os.path.exists(f"{const.root}/settings.json"):
        f = open(f"{const.root}/settings.json", 'x')
        f.write(default_settings)
        f.close()
        
    elif replace:
        f = open(f"{const.root}/settings.json", 'w')
        f.write(default_settings)
        f.close()

    if not os.path.exists(f"{const.root}/config"):
        os.mkdir(f"{const.root}/config")



def build_reference_directories(folder=f'{const.root}/ref'):
    """Builds directories for reference.

    Args:
        folder (str, optional): Path of the reference directory to build subdirectories. Defaults to 'test/asm/ref'.
    """

    if not os.path.exists(folder):
        os.mkdir(f"{folder}")

    for c in const.COMPILER:
        if not os.path.exists(f"{folder}/{c}"):
            os.mkdir(f"{folder}/{c}")

        for a in const.ARCH:
            if not os.path.exists(f"{folder}/{c}/{a}"):
                os.mkdir(f"{folder}/{c}/{a}")


def reset(folder=f'{const.root}/ref'):
    """Clear directory

    Args:
        folder (str, optional): Path of the directory. Defaults to 'test/asm/ref'.
    """
    os.system(f"rm -rf {folder}/*")



if __name__ == '__main__':
    pass
    #build_reference_directories()